-- incldep.ex -- include file dependency checker
-- can list include files used by a program and its include files
-- or it can list exact dependencies between include files, such as:
-- file X is used by file Y
-- file Y is used by file Z
-- incldep.ex will tell you when files are missing

include strtok.e
include get.e
include misc.e

sequence tlist,ilist,dlist,tdlist,incl,cmd,missing,levels
integer full,SLASH_CHAR,level
object eudir,euinc
level   = 0
levels  = {}
incl    = {}
tlist   = {} tdlist = {}
ilist   = {} dlist  = {}
missing = {}
if platform()=LINUX then
    SLASH_CHAR='/'
else
    SLASH_CHAR='\\'
end if

eudir=getenv("EUDIR")
euinc=getenv("EUINC")
-- add directories from EUINC, if present, to search path
if sequence(euinc) then
    euinc=parse(euinc,';')
    for i = 1 to length(euinc) do
	if euinc[i][length(euinc[i])] != SLASH_CHAR then
	    euinc[i] &= SLASH_CHAR
	end if
	incl=append(incl,euinc[i])
    end for
end if
-- add the INCLUDE directory from the system's EUDIR, if present, to search path
if sequence(eudir) then
    if eudir[length(eudir)] != SLASH_CHAR then
	eudir &= SLASH_CHAR
    end if
    incl=append(incl,eudir&"include"&SLASH_CHAR)
end if

procedure check(sequence name)
    sequence fname
    object ln
    integer fn,comm
    level += 1
    -- if on Windows or DOS, ignore case
    if platform()=WIN32 or platform()=DOS32 then
	name=lower(name)
    end if
    fn=open(name,"r")
    if fn=-1 then
	-- not found in current directory. try search path
	for i = 1 to length(incl) do
	    fn=open(incl[i]&name,"r")
	    if fn != -1 then exit end if
	end for
    end if
    if fn=-1 then
	-- not found in current directory or search path. put down as missing
	-- and cancel this level of the search
	if level > 1 then
	    if not find(name,missing) then
		missing=append(missing,name)
	    end if
	else
	    puts(1,"not found\n")
	    abort(1)
	end if
	return
    end if
    while 1 do
	ln=gets(fn)
	if atom(ln) then
	    exit
	end if
	-- remove trailing newline
	if find('\n',ln) then
	    ln=ln[1..length(ln)-1]
	end if
	-- remove any white space at the beginning
	for i = 1 to length(ln) do
	    if not find(ln[i]," \t") then
		ln=ln[i..length(ln)]
		exit
	    end if
	end for
	-- valid include keyword found?
	if match("include",ln) and equal(gettok(ln,1,' '),"include") and
	   numtok(ln,' ') >= 2 then
	    -- get filename
	    fname=ln[find(' ',ln)+1..length(ln)]
	    -- remove whitespace from the beginning
	    for i = 1 to length(fname) do
		if not find(fname[i]," \"\t") then
		    fname=fname[i..length(fname)]
		    exit
		end if
	    end for
	    if match("--",fname) then
		comm=2
	    else
		comm=0
	    end if
	    -- remove white space from end
	    for i = length(fname) to 1 by -1 do
		-- hit the beginning of the white space
		-- if still in a comment, ignore this
		if not find(fname[i]," \"\t") and comm=0 then
		    fname=fname[1..i]
		    exit
		elsif fname[i]='-' then
		    -- hit a dash. one step closer to the end of the
		    -- comment
		    comm -= 1
		    fname=fname[1..i-1]
		end if
		-- of course, double dashes can be used and sort of
		-- "masquerade" as comments. check for more
		if comm=0 and match("--",fname) then
		    comm=2
		end if
	    end for
	    -- check for namespace keyword "as" and remove it if present
	    if numtok(fname,' ')=3 and equal(gettok(fname,2,' '),"as") then
		fname=fname[1..find(' ',fname)-1]
	    end if
	    -- check the file for more used include files
	    if full then
		if not find({fname,name},tlist) then
		    tlist=append(tlist,{fname,name})
		    levels &= level
		    check(fname)
		end if
	    else
		if not find(fname,tlist) then
		    tlist=append(tlist,fname)
		    check(fname)
		end if
	    end if
	end if
    end while
    close(fn)
    level -= 1
end procedure

puts(1,"include file dependency checker v1.0a by Matt Arriola\n")
cmd=command_line()
if length(cmd)=2 then
    cmd=append(cmd,prompt_string("Start from: "))
end if
puts(1,"Do full dependency listing? ")
while 1 do
    full=wait_key()
    if find(full,"YyNn") then
	puts(1,full&'\n')
	exit
    end if
end while
full=find(full,"Yy")
check(cmd[3]) -- check starter file
-- remove duplicate entries
for i = 1 to length(tlist) do
    if full or not find(tlist[i],ilist) then
	ilist=append(ilist,tlist[i])
    end if
end for

if full then
    for i = 1 to length(ilist) do
	puts(1,repeat(' ',(levels[i]-1)*2))
	printf(1,"`%s' is used by `%s'\n",ilist[i])
    end for
else
    for i = 1 to length(ilist) do
	printf(1,"`%s'\n",{ilist[i]})
    end for
end if
for i = 1 to length(missing) do
    printf(1,"`%s' is needed, but not present\n",{missing[i]})
end for
